-module(mrend).
%-compile(export_all).
-export([map2/2, filter2/2, foldr/3, foldl/3]).
-author('kapolnai@iit.bme.hu, hanak@emt.bme.hu').
-vsn('$LastChangedDate: 2019-10-29 21:47:55 +0100 (k, 29 okt 2019) $$').

-spec map2(F::fun((X::term()) -> Y::term()), Xs::list(term())) -> Ys::list(term()). 
% Az Xs lista X elemeiből az F transzformációval képzett Y elemek listája Ys.
map2(_F, []) ->
    [];
map2(F, [X|Xs]) ->
    [F(X) | map2(F, Xs)].

-spec filter2(P::fun((X::term()) -> B::boolean()), Xs::[term()]) -> Ys::[term()].
% Az Xs lista X elemeinek P-t kielégítő listája Ys.
filter2(_, []) ->
    [];
filter2(P, [Fej|Farok]) ->
    case P(Fej) of
	true -> [Fej|filter2(P,Farok)];
	false -> filter2(P,Farok)
    end.

-spec foldr(F::fun((X::term(), E::term()) -> E1::term()), E0::term(), Xs::[term()]) -> R::term().
% Az E0-ból és az Xs listában jobbról balra haladva az Xs elemeiből a kétoperandusú
% F-fel képzett érték R.
foldr(_Fun, Acc, [ ]) ->
    Acc;
foldr(Fun, Acc, [X|Xs]) ->
    Fun(X, foldr(Fun, Acc, Xs)).

-spec foldl(F::fun((X::term(), E::term()) -> E1::term()), E0::term(), Xs::[term()]) -> R::term().
% Az E0-ból és az Xs listában balról jobbra haladva az Xs elemeiből a kétoperandusú
% F-fel képzett érték R.
foldl(_Fun, Acc, [ ]) ->
    Acc;
foldl(Fun, Acc, [X|Xs]) ->
    foldl(Fun, Fun(X, Acc), Xs).
